/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package core;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.geom.Ellipse2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.concurrent.ThreadLocalRandom;
import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

/**
 *
 * @author SUPERMICRO USER
 */
public class Game {

   JLabel[] lineLabels;
   JLabel[] scoreLabels;
   JLabel winnerLabel;

   String[][] rows = new String[12][6];

   String[] winningLine = new String[6];

   String colorLetters[] = {"r","g","b","y"};

   int currentLabelIndex;
   public JLabel currentLabel;

   int level = 4;
   public boolean gameOver = false;

   BufferedImage plainImage = null;
   BufferedImage redImage = null;
   BufferedImage greenImage = null;
   BufferedImage blueImage = null;
   BufferedImage yellowImage = null;

   BufferedImage newImage = null;
   BufferedImage winningImage = null;

   Ellipse2D.Double[] circles = {new Ellipse2D.Double(2, 2, 16, 16),
                                    new Ellipse2D.Double(22, 2, 16, 16),
                                    new Ellipse2D.Double(42, 2, 16, 16),
                                    new Ellipse2D.Double(62, 2, 16, 16),
                                    new Ellipse2D.Double(82, 2, 16, 16),
                                    new Ellipse2D.Double(102, 2, 16, 16)};

   public Game(JLabel[] labels1, JLabel[] labels2, JLabel winningOutput){
       this.lineLabels = labels1;
       this.scoreLabels = labels2;
       this.winnerLabel = winningOutput;

       try
            {
                this.plainImage = ImageIO.read(Game.class.getResourceAsStream("/resources/1.png"));
                this.redImage = ImageIO.read(Game.class.getResourceAsStream("/resources/2.png"));
                this.greenImage = ImageIO.read(Game.class.getResourceAsStream("/resources/3.png"));
                this.blueImage = ImageIO.read(Game.class.getResourceAsStream("/resources/4.png"));
                this.yellowImage = ImageIO.read(Game.class.getResourceAsStream("/resources/5.png"));
            }
            catch (IOException ex)
            {
                ex.printStackTrace();
                return;
            }
       newGame();
   }

   public void setLevel(int aLevel){
       this.level = aLevel;
       newGame();
   }

   public final void newGame(){
       for(int y = 0; y < 12; y++){
           for(int x = 0; x < 6; x++){
               this.rows[y][x] = "";
           }
       }

       for(int x = 0; x < 6; x++){
          if(x < this.level){
              int i = ThreadLocalRandom.current().nextInt(0, 4);
              this.winningLine[x] = colorLetters[i];
          }
          else {
              this.winningLine[x] = "";
          }
       }

       BufferedImage image = null;
       ImageIcon ii = null;

       for(JLabel lbl : this.lineLabels){
            try
            {
                image = ImageIO.read(Game.class.getResourceAsStream("/resources/1.png"));
                ii = new ImageIcon(image);
            }
            catch (IOException ex)
            {
                ex.printStackTrace();
            }
            lbl.setIcon(ii);
       }

       this.winnerLabel.setIcon(ii);
       this.winnerLabel.setSize(80, 20);

       for(JLabel lbl : this.scoreLabels){
            lbl.setIcon(null);
       }

       this.currentLabelIndex = this.level * 2 - 1;
       this.currentLabel = this.lineLabels[this.currentLabelIndex];

       newImage = copyImage(this.plainImage);
       winningImage  = copyImage(this.plainImage);
       drawImage(winningImage.getGraphics(), this.winningLine);

       gameOver = false;
   }

   public boolean play(Point p){
        Graphics g = newImage.getGraphics();
        int counter = 0;
        for(int x = 0; x < this.level; x++){
            if(circles[x].contains(p)){
                if(this.rows[this.currentLabelIndex][x].equals("")){
                    this.rows[this.currentLabelIndex][x] = "r";
                }
                else {
                    for(int i = 0; i < 4; i++){
                        if(this.rows[this.currentLabelIndex][x].equals(this.colorLetters[i])){
                            this.rows[this.currentLabelIndex][x] = this.colorLetters[(i + 1) % 4];
                            break;
                        }
                    }
                }
                
                break;
            }
            
        }

        for(int x = 0; x < this.level; x++){
            if(!this.rows[this.currentLabelIndex][x].equals("")){
                    counter++;
            }
        }


        drawImage(g, this.rows[this.currentLabelIndex]);

        this.currentLabel.setIcon(new ImageIcon(newImage));

        return counter == this.level;

   }

   private void drawImage(Graphics g, String[] array){
       for(int x = 0; x < this.level; x++){
            if(array[x].equals("r")){
                g.drawImage(this.redImage, x * 20, 0, 20, 20, null);
            }
            else if(array[x].equals("g")){
                g.drawImage(this.greenImage, x * 20, 0, 20, 20, null);
            }
            else if(array[x].equals("b")){
                g.drawImage(this.blueImage, x * 20, 0, 20, 20, null);
            }
            else if(array[x].equals("y")){
                g.drawImage(this.yellowImage, x * 20, 0, 20, 20, null);
            }
        }
   }

   private BufferedImage copyImage(BufferedImage source){
        BufferedImage b = new BufferedImage(source.getWidth(), source.getHeight(), source.getType());
        Graphics g = b.getGraphics();
        g.drawImage(source, 0, 0, null);
        g.dispose();
        return b;
    }

   public void checkLine(){
        char[] tempWinningLine = new char[this.level];
        char[] tempCurrentLine = new char[this.level];

        for(int x = 0; x < this.level; x++){
            tempWinningLine[x] = this.winningLine[x].charAt(0);
            tempCurrentLine[x] = this.rows[this.currentLabelIndex][x].charAt(0);
        }



        int redCount = 0;
        int blackCount = 0;
               
        for(int x = 0; x < this.level; x++){
            if(tempWinningLine[x] == tempCurrentLine[x]){
                 blackCount++;
                 tempWinningLine[x] = 'z';
                 tempCurrentLine[x] = 'z';
            }
        }

     
        for(int x = 0; x < this.level; x++){
            if(tempWinningLine[x] != 'z'){
                for(int y = 0; y < this.level; y++){
                    if(tempWinningLine[x] == tempCurrentLine[y]){
                        redCount++;
                        tempCurrentLine[y] = 'z';
                    }
                }
            }
        }

        
        BufferedImage scoreImage = new BufferedImage(30,20, BufferedImage.TRANSLUCENT);
        Graphics g = scoreImage.getGraphics();

        int startX = -5;
        int startY = 11;
        for(int x = 0; x < this.level; x++){
            if(x == 0 ||x == 2 || x == 4){
               startX += 9;
               startY = 11;
            }
            if(x < blackCount){
                g.setColor(Color.black);
                g.fillOval(startX, startY, 5, 5);
            }
            else if(x < blackCount + redCount){
                g.setColor(Color.red);
                g.fillOval(startX, startY, 5, 5);
            }
            g.setColor(Color.black);
            g.drawOval(startX, startY, 5, 5);
            startY -= 7;
        }

        this.scoreLabels[this.currentLabelIndex].setIcon(new ImageIcon(scoreImage));

        newImage = copyImage(this.plainImage);

        if(blackCount != this.level && this.currentLabelIndex > 0){
            this.currentLabelIndex -= 1;
            this.currentLabel = this.lineLabels[this.currentLabelIndex];
        }
        else if(blackCount == this.level){
            this.winnerLabel.setIcon(new ImageIcon(winningImage));
            this.gameOver = true;
            JOptionPane.showMessageDialog(currentLabel, "You won!");
        }
        else {
            this.winnerLabel.setIcon(new ImageIcon(winningImage));
            this.gameOver = true;
            JOptionPane.showMessageDialog(currentLabel, "You lost");
        }
    }

}

